import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import javax.swing.*;
import java.awt.event.*;
import java.lang.*;

/**
   A panel that draws a bar chart.
   @version 1.31 2002-07-06
   @author Cay Horstmann modified by Eduardo Gonzalez Fisher.
*/
class ChartPanel extends JPanel
{
    /**
     * Constructs a ChartPanel.
     * @param v the array of values for the chart
     * @param m the array of names for the values
     * @param maxC Cota maxima del grafico
     */
    public ChartPanel (double[] v, String[] m, int maxC)
    {
        maxCorrectas = maxC;
        names = m;
        values = v;
        title = "Tarea 3";
        ultima = v.length - 1;
    }
    /**
     * Metodo encargado de dibujar la pantalla.
     * Es aqui donde se crea y dibuja el grafico.
     */
    public void paintComponent (Graphics g)
    {
        final int TAM_20 = 20;
        final int TAM_10 = 10;
        Font titleFont = new Font ("SansSerif", Font.BOLD, TAM_20);
        Font labelFont = new Font ("SansSerif", Font.PLAIN, TAM_10);
        super.paintComponent (g);
        Graphics2D g2 = (Graphics2D) g;
        double minValue = 0;
        double maxValue = maxCorrectas;
        int panelWidth = getWidth ();
        int panelHeight = getHeight ();
        FontRenderContext context = g2.getFontRenderContext ();
        Rectangle2D titleBounds = titleFont.getStringBounds (title, context);
        double titleWidth = titleBounds.getWidth ();
        double top = titleBounds.getHeight ();
        double y = -titleBounds.getY ();
        double x = (panelWidth - titleWidth) / 2;
        g2.setFont (titleFont);
        g2.drawString (title, (float) x, (float) y);
        LineMetrics labelMetrics  = labelFont.getLineMetrics ("", context);
        double bottom = labelMetrics.getHeight ();
        y = panelHeight - labelMetrics.getDescent ();
        g2.setFont (labelFont);
        // get the scale factor and width for the bars
        double scale = (panelHeight - top - bottom) / (maxValue - minValue);
        int barWidth = panelWidth / values.length;
        // draw the bars
        for (int i = 0; i < values.length; i++)
        {   // get the coordinates of the bar rectangle
            double x1 = i * barWidth + 1;
            double y1 = top;
            double height = values[i] * scale;
            if (values[i] >= 0)
            y1 += (maxValue - values[i]) * scale;
            else
            {
                y1 += maxValue * scale;
                height = -height;
            }
            // rellena las barras y dibuja el contorno
            Rectangle2D rect = new Rectangle2D.Double (x1, y1, barWidth - 2,
                                                       height);
            g2.setPaint (Color.RED);
            g2.fill (rect);
            g2.setPaint (Color.BLACK);
            g2.draw (rect);
            // draw los rotulos
            Rectangle2D labelBounds = labelFont.getStringBounds (names[i],
                                                                 context);
            double labelWidth = labelBounds.getWidth ();
            x = i * barWidth + (barWidth - labelWidth) / 2;
            g2.drawString (names[i], (float) x, (float) y);
        }
    }
    /**
     * Actualiza el grafico.
     * @param valor Nuevo valor.
     * @param rotulo Rotulo del valor
     */
    public void nuevaMovida(int valor, int rotulo)
    {
        /*
        int[] temp = new int[values.length + 1]
        int[] aux = new int[names.length + 1]
        for (int i = 0; i < values.length; i++)
        {
            temp[i] = values[i];
            aux[i] = names[i];
        }*/
        values = desplazarArreglo(values);
        values[ultima] = valor;
        names = desplazarArreglo(names);
        names[ultima] = Integer.toString(rotulo);
        repaint ();
    }
    
    /**
     * Desplaza los valores dentro de un arreglo.
     * @param a Arreglo.
     */     
    private double[] desplazarArreglo(double[]a)
    {
        for (int i = 1; i < a.length; i++)
        {
            a[i - 1] = a[i];
        }
        return a;
    }
    /**
     * Desplaza los valores dentro de un arreglo.
     * @param a Arreglo.
     */
    private String[]desplazarArreglo(String[]a)
    {
        for (int i = 1; i < a.length; i++)
        {
            a[i - 1] = a[i];
        }
        return a;
    }
    /** es el indice al ultimo elemento a graficar */
    private final int ultima;
    /** arreglo con los valores del grafico */
    private double[] values;
    /** arreglo con los rotulos del grafico */
    private String[]names;
    /** Titulo del Grafico */
    private String title;
    /** Cota superior del grafico */
    private int maxCorrectas;
}

