import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.lang.*;
import java.io.*;
import javax.swing.*;

/**
 * Puzzle: Un tipico juego de Puzzle.
 * @author Juan J. Sierralta, modified by Eduardo Gonzalez.
 */
public class Puzzle extends Canvas implements MouseListener
{
  /**
   * Crea el puzzle.
   * @param im Imagen del puzzle.
   * @param gr Grafico de las jugadas.
   * @param hor Numero de divisiones horizontales.
   * @param vert Numero de divisiones verticales.
   */
  public Puzzle (Image im, ChartPanel gr, int hor, int vert)
  {
    grafo = gr;
    addMouseListener (this); // Escuchamos eventos de mouse
    try
    {
      imagen = im;
      MediaTracker t = new MediaTracker (this);
      t.addImage (imagen, 0);
      t.waitForID (0);
      iw = imagen.getWidth (null);
      ih = imagen.getHeight (null);
      v = vert;
      h = hor;
      iniciarVariables ();
      celda = new Image[numceldas]; // Ahora creamos numceldas subimagenes
      orden = new int[numceldas];
      CropImageFilter f;
      FilteredImageSource fis;
      for (int y = 0; y < v; y++)
        {
          for (int x = 0; x < h; x++)
            {
              f = new CropImageFilter (cw * x, ch * y, cw, ch);
              fis = new FilteredImageSource (imagen.getSource (), f);
              int i = y * h + x;
              orden[i] = i;
              celda[i] = createImage (fis);
              t.addImage (celda[i], i);
            }
        }
      t.waitForAll ();
      shuffle ();               // Barajamos las subimagenes
    }
    catch (Exception e)
    {
      System.out.println ("Ha ocurrido una excepcion: " + e.getMessage ());
    }
  }
  /**
   * Baraja las subimagenes.
   */
  private void shuffle ()
  {
    for (int i = 0; i < numceldas * 2; i++)
      {
        int si = (int) (Math.random () * numceldas);
        int di = (int) (Math.random () * numceldas);
        int tmp = orden[si];
        orden[si] = orden[di];
        orden[di] = tmp;
      }
  }

  /**
   * Calcula el numero de celdas bien posicionadas.
   * @return numero de piezas correctas.
   */
  public int puntaje ()
  {
    int score = 0;
    for (int i = 0; i < numceldas; i++)
      {
        if (orden[i] == i)
          score++;
      }
    return score;
  }

  /**
   * Se llama cuando se hace un click de mouse.
   * Se encarga de realizar las operaciones de mover las piezas,
   * cuando corresponda.
   * @param e evento -click del mouse-
   */
  public void mouseClicked (MouseEvent e)
  {
    final int Y_OFFSET = 50;
    boolean mismaPieza;
    int x = e.getX ();
    int y = e.getY () - Y_OFFSET;
    x = x / (cw + 1);
    y = y / (ch + 1);
    int pos = y * h + x;
    mismaPieza = (pos != lastPos) ? false : true;
    if (!mismaPieza && !solved)
      {
        if (((pos % h) != 0) && (orden[pos - 1] == last))       //Izquierda
          {
            orden[pos - 1] = orden[pos];
            orden[pos] = last;
            movimiento (pos);
          }
        else if (((pos - h) >= 0) && (orden[pos - h] == last))  //Arriba
          {
            orden[pos - h] = orden[pos];
            orden[pos] = last;
            movimiento (pos);
          }
        else if (((lastPos % h) != 0) && (orden[pos + 1] == last)) //Derecha
          {
            orden[pos + 1] = orden[pos];
            orden[pos] = last;
            movimiento (pos);
          }
        else if (((pos + h) <= last) && (orden[pos + h] == last))
        //Abajo
          {
            orden[pos + h] = orden[pos];
            orden[pos] = last;
            movimiento (pos);
          }
        movStr = "Movidas: " + movidas;
      }
  }
    /**
     * Codigo relacionado con el movimiento de una pieza
     * @param pos posicion de la pieza
     */
  private void movimiento (int pos)
  {
    lastPos = pos;
    movidas++;
    grafo.nuevaMovida (puntaje (), movidas);
    repaint ();
  }


    /**
   * No se usa.
   * @param e Evento del mouse.
   */
  public void mousePressed (MouseEvent e)
  {
  }
    /**
   * No se usa.
   * @param e Evento del mouse.
   */
  public void mouseReleased (MouseEvent e)
  {
  }
    /**
   * No se usa.
   * @param e Evento del mouse.
   */
  public void mouseEntered (MouseEvent e)
  {
  }
    /**
   * No se usa.
   * @param e Evento del mouse.
   */
  public void mouseExited (MouseEvent e)
  {
  }

    /**
   * Repinta el componente, llama a paint.
   * @param g Componente.
   */
  public void update (Graphics g)
  {
    paint (g);
  }

    /**
   * Aqui se dibuja el componente.
   * @param g Componente.
   */
  public void paint (Graphics g)
  {
    final int TAM_RECT = 50;
    final int POS_X_MOV = 5;
    final int POS_Y_MOV = 12;
    // Borramos el texto anterior
    backCol = Color.black;
    g.setColor (backCol);
    g.fillRect (0, 0, TAM_RECT + TAM_RECT + TAM_RECT, TAM_RECT);
    backCol = Color.cyan;
    g.setColor (backCol);
    // Imprimimos el numero de movidas
    g.drawString (movStr, POS_X_MOV, POS_Y_MOV);
    // Obtenemos el puntaje y lo mostramos
    int score = puntaje ();
    if (score == (v * h))
      {                         // Ganamos !
        solved = true;
        g.setColor (Color.yellow);
        g.drawString ("Felicitaciones !", POS_X_MOV, POS_Y_MOV + POS_Y_MOV);
      }
    else
      { // Mostramos el puntaje (numero de piezas bien colocadas)
        g.drawString (score + " piezas correctas", POS_X_MOV,
                      POS_Y_MOV + POS_Y_MOV);
      }
    // Hacemos un rectangulo cyan detras de las subimagenes asi,
    // separando cada celda por 1 pixel de sus vecinas se ve una grilla.
    g.fillRect (0, TAM_RECT, iw - 1 + h - 1, ih + v);
    // Dibujamos cada subimagen
    for (int y = 0; y < v; y++)
      {
        for (int x = 0; x < h; x++)
          {
            if (orden[y * h + x] != last)
              g.drawImage (celda[orden[y * h + x]], (x * cw) + x,
                           TAM_RECT + (y * ch) + y, this);
          }
      }
  }
  /**
   * Inicia atributos del objeto Puzzle.
   */
  private void iniciarVariables ()
  {
    movidas = 0;
    solved = false;
    movStr = "Movidas: 0";
    cw = iw / h;
    ch = ih / v;
    numceldas = v * h;
    last = numceldas - 1;
    lastPos = last;
  }

  /** cantidad vertical de sub-imagenes */
  private int v;
  /** cantidad vertical de sub-imagenes */
  private int h;
  /** ancho de la imagen */
  private int iw;
  /** alto de la imagen */
  private int ih;
  /** ancho de la sub-imagen */
  private int cw;
  /** alto de la sub-imagen */
  private int ch;
  /** Numero de jugadas */
  private int movidas;
  /** Sub-Imagen vacia, la que se desplaza */
  private int last;
  /** Numero de celdas o sub-imagenes */
  private int numceldas;
  /** Puzzle resuelto o inconcluso? */
  private boolean solved;
  /** Arreglo de imagenes */
  private Image[] celda;
  /** Orden de las imagenes */
  private int[] orden;
  /** Posicion del cuadro vacio */
  private int lastPos;
  /** Imagen del puzzle */
  private Image imagen;
  /** String que indica las jugadas */
  private String movStr;
  /** Grafico de las jugadas */
  private ChartPanel grafo;
  /** un color */
  private Color backCol;
}



